#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <rsbac/types.h>
#include <rsbac/aci_data_structures.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include "locale.h"

int main(int argc, char ** argv)
{
  enum rsbac_attribute_t attr_list[RSBAC_FD_NR_ATTRIBUTES] = RSBAC_FD_ATTR_LIST;
  enum rsbac_attribute_t attr_list_dev[RSBAC_DEV_NR_ATTRIBUTES] = RSBAC_DEV_ATTR_LIST;
  int res = 0;
  char tmp1[80],tmp2[256];
  int j;
  u_int position,catval;
  union rsbac_attribute_value_t value,value2;
  enum rsbac_target_t target;
  enum rsbac_attribute_t attr;
  enum rsbac_adf_request_t request;
  enum rsbac_log_level_t log_level;
  __u64 k;
  char * progname;
  int verbose = 0;
  int printall = 0;
  int add = 0;
  int remove = 0;

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);

  progname = argv[0];
  while((argc > 1) && (argv[1][0] == '-'))
    {
      char * pos = argv[1];
      pos++;
      while(*pos)
        {
          switch(*pos)
            {
              case 'v':
                verbose++;
                break;
              case 'p':
                printall=1;
                break;
              case 'a':
                add=1;
                break;
              case 'm':
                remove=1;
                break;
              case 'n':
                {
                  char tmp[80];
                  int i;

                  for(i=0; i<R_NONE; i++)
                    printf("%s\n", get_request_name(tmp, i));
                  exit(0);
                }
              default:
                fprintf(stderr, gettext("%s: unknown parameter %c\n"), progname, *pos);
            }
          pos++;
        }
      argv++;
      argc--;
    }

  if(   (argc>3)
     && !strcmp(argv[3],"log_program_based")
    )
    {
      char * filename = argv[2];
      boolean rused = FALSE;
      boolean wused = FALSE;
      enum rsbac_adf_request_t request;
      rsbac_request_vector_t request_vector = 0;

      target = get_target_nr(argv[1]);
      value.log_program_based = 0;
      argv+=3;
      argc-=3;
      if(add || remove)
        {
          res = rsbac_get_attr_n(target, filename, A_log_program_based, &value, FALSE);
          error_exit(res);
        }
      while(argc > 1)
        {
          if(strlen(argv[1]) == R_NONE)
            {
              int j;
              rsbac_request_vector_t tmp_rv;

              for(j=0; j<R_NONE; j++)
                if(   (argv[1][j] != '0')
                   && (argv[1][j] != '1')
                  )
                  {
                    fprintf(stderr, gettext("%s: Invalid mask vector %s\n"), progname, argv[1]);
                    exit(1);
                  }
              strtou64acl(argv[1], &tmp_rv);
              request_vector |= tmp_rv;
              argv++;
              argc--;
              continue;
            }
          request = get_request_nr(argv[1]);
          if(request == R_NONE)
            {
              request = strtol(argv[1],0,10);
              if(   (request >= R_NONE)
                 || (   (request == 0)
                     && strcmp(argv[1],"0")
                    )
                )
                {
                  if(!strcmp(argv[1],"RW"))
                    {
                      request_vector |= RSBAC_READ_WRITE_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"SY"))
                    {
                      request_vector |= RSBAC_SYSTEM_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"SE"))
                    {
                      request_vector |= RSBAC_SECURITY_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"R"))
                    {
                      request_vector |= RSBAC_READ_REQUEST_VECTOR;
                      rused = TRUE;
                    }
                  else
                  if(!strcmp(argv[1],"W"))
                    {
                      request_vector |= RSBAC_WRITE_REQUEST_VECTOR;
                      wused = TRUE;
                    }
                  else
                  if(!strcmp(argv[1],"A"))
                    {
                      request_vector |= RSBAC_ALL_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"UA"))
                    {
                      request_vector = 0;
                    }
                  else
                  if(!strcmp(argv[1],"NWS"))
                    {
                      request_vector |= RSBAC_NWS_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"NWR"))
                    {
                      request_vector |= RSBAC_NWR_REQUEST_VECTOR;
                      rused = TRUE;
                    }
                  else
                  if(!strcmp(argv[1],"NWW"))
                    {
                      request_vector |= RSBAC_NWW_REQUEST_VECTOR;
                      wused = TRUE;
                    }
                  else
                  if(!strcmp(argv[1],"NWC"))
                    {
                      request_vector |= RSBAC_NWC_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"NWE"))
                    {
                      request_vector |= RSBAC_NWE_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"NWA"))
                    {
                      request_vector |= RSBAC_NWA_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"NWF"))
                    {
                      request_vector |= RSBAC_NWF_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"NWM"))
                    {
                      request_vector |= RSBAC_NWM_REQUEST_VECTOR;
                    }
                  else
                    { /* end of requests */
                      break;
                    }
                }
            }
          else
            {
              request_vector |= ((rsbac_request_vector_t) 1 << request);
            }
          argv++;
          argc--;
        }
      if(rused && wused)
        {
          request_vector |= RSBAC_READ_WRITE_OPEN_REQUEST_VECTOR;
        }
      if(remove)
          value.log_program_based &= ~request_vector;
      else
          value.log_program_based |= request_vector;

      if(printall)
        {
          int i;

          for (i=0; i<R_NONE; i++)
            if(value.log_program_based & ((rsbac_request_vector_t) 1 << i))
              printf(" %s\n", get_request_name(tmp1,i));
        }
      res = rsbac_set_attr_n(target, filename, A_log_program_based, &value);
      error_exit(res);
      exit(0);
    }

  switch(argc)
    {
      case 5:
        target = get_target_nr(argv[1]);
        attr = get_attribute_nr(argv[3]);
        switch(attr)
          {
            case A_none:
              fprintf(stderr, gettext("%s: Invalid attribute %s\n"), argv[0], argv[3]);
              exit(1);
            case A_log_array_low:
            case A_log_array_high:
            case A_log_program_based:
              if(strlen(argv[4]) != R_NONE)
                {
                  fprintf(stderr, gettext("%s: Invalid attribute value, length must be %i\n"),
                                  argv[0], R_NONE);
                  exit(1);
                }
              for(j=0;j<strlen(argv[4]);j++)
                {
                  if(   argv[4][j] != '0'
                     && argv[4][j] != '1')
                    {
                      fprintf(stderr, gettext("%s: Invalid attribute value char, must be 0 or 1\n"), argv[0]);
                      exit(1);
                    }
                }
              strtou64log(argv[4], &value.log_array_low);
              break;

            case A_mac_categories:
              if(strlen(argv[4]) != RSBAC_MAC_NR_CATS)
                {
                  fprintf(stderr, gettext("%s: Invalid attribute value, length must be %i\n"), argv[0],
                          RSBAC_MAC_NR_CATS);
                  exit(1);
                }
              for(j=0;j<strlen(argv[4]);j++)
                {
                  if(   argv[4][j] != '0'
                     && argv[4][j] != '1')
                    {
                      fprintf(stderr, gettext("%s: Invalid attribute value char, must be 0 or 1\n"), argv[0]);
                      exit(1);
                    }
                }
              strtou64mac(argv[4], &value.mac_categories);
              break;

            case A_mac_trusted_for_user:
              value.mac_trusted_for_user = strtoul(argv[4],0,10);
              break;

            default:
              value.dummy = strtol(argv[4],0,10);
          }
        if(verbose)
          {
            printf(gettext("Setting attribute %s for %s to value %lu\n"),
                   get_attribute_name(tmp1,attr),
                   argv[2],
                   value.u_long_dummy);
          }
        res = rsbac_set_attr_n(target, argv[2], attr, &value);
        error_exit(res);
        exit(0);

      case 6:
        if(!strcmp(argv[3],"log_level"))
          {
            target = get_target_nr(argv[1]);
            request = get_request_nr(argv[4]);
            if(request == R_NONE)
              {
                fprintf(stderr, gettext("Invalid request type %s\n"), argv[4]);
                printf(gettext("Valid request types:\n"));
                for(j=0;j<R_NONE;j++)
                  {
                    printf("%s\n", get_request_name(tmp1,j));
                  }
                exit(1);
              }
            log_level = get_log_level_nr(argv[5]);
            if(log_level == LL_invalid)
              {
                log_level = strtol(argv[5],0,10);
                if(   (log_level >= LL_invalid)
                   || (   (log_level == LL_none)
                       && strcmp(argv[5], "0")) )
                  {
                    fprintf(stderr, gettext("Invalid log_level value %s\n"), argv[5]);
                    exit(1);
                  }
              }
            res = rsbac_get_attr_n(target, argv[2], A_log_array_low, &value, FALSE);
            error_exit(res);
            res = rsbac_get_attr_n(target, argv[2], A_log_array_high, &value2, FALSE);
            error_exit(res);
            k = ((__u64) 1) << request;
            if(log_level & 1)
              value.log_array_low |= k;
            else
              value.log_array_low &= ~k;
            if(log_level & 2)
              value2.log_array_high |= k;
            else
              value2.log_array_high &= ~k;
            res = rsbac_set_attr_n(target, argv[2], A_log_array_low, &value);
            error_exit(res);
            res = rsbac_set_attr_n(target, argv[2], A_log_array_high, &value2);
            error_exit(res);
            exit(0);
          }
        else
        if(!strcmp(argv[3],"mac_categories"))
          {
            target = get_target_nr(argv[1]);
            position = strtol(argv[4],0,10);
            if(position > RSBAC_MAC_MAX_CAT)
              {
                fprintf(stderr, gettext("Invalid position counter %s\n"), argv[4]);
                exit(1);
              }
            catval = strtol(argv[5],0,10);
            if(catval > 1)
              {
                fprintf(stderr, gettext("Invalid value %s\n"), argv[5]);
                exit(1);
              }
            res = rsbac_get_attr_n(target, argv[2], A_mac_categories, &value, FALSE);
            error_exit(res);
            k = ((__u64) 1) << position;
            if(catval)
              value.mac_categories |= k;
            else
              value.mac_categories &= ~k;
            res = rsbac_set_attr_n(target, argv[2], A_mac_categories, &value);
            error_exit(res);
            exit(0);
          }

      default:
        printf(gettext("%s (RSBAC %s)\n***\n"), argv[0], VERSION);
        printf(gettext("Use: %s target-type file/dirname attribute [request] value\n"), argv[0]);  
        printf(gettext("Use: %s target-type file/dirname attribute [position] value\n"), argv[0]);  
        printf(gettext("Use: %s [switches] target-type filename log_program_based [list-of-requests]\n"), argv[0]);  
        printf(gettext("switches: -a = add, not set, -m = remove not set, -p = print resulting requests,\n"));
        printf(gettext("- target-type = FILE, DIR, FIFO or DEV,\n"));
        printf(gettext("- attribute (string) and value (integer) = see following list:\n"));
        printf(gettext("- FILE, DIR and FIFO:\n"));
        printf(gettext("log_level\t\t(additional parameter request-type)\n\t\t\t0=none, 1=denied, 2=full, 3=request-based\n"));
        printf(gettext("mac_categories\t\t(with additional parameter position)\n\t\t\t0=no, 1=yes\n"));
        for (j=0;j<RSBAC_FD_NR_ATTRIBUTES;j++)
          {
            get_attribute_name(tmp1,attr_list[j]);
            get_attribute_param(tmp2,attr_list[j]);
            printf("%s \t%s\n",tmp1,tmp2);
          }
        printf("- DEV:\n");
        printf(gettext("log_level\t\t(additional parameter request-type)\n\t\t\t0=none, 1=denied, 2=full, 3=request based\n"));
        printf(gettext("mac_categories\t\t(with additional parameter position)\n\t\t\t0=no, 1=yes\n"));
        for (j=0;j<RSBAC_DEV_NR_ATTRIBUTES;j++)
          {
            get_attribute_name(tmp1,attr_list_dev[j]);
            get_attribute_param(tmp2,attr_list_dev[j]);
            printf("%s \t%s\n",tmp1,tmp2);
          }
      }
  exit(1);
}
