#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <curses.h>

#include "r-job.h"
#include "r-job-lib.h"

#define SLEEP_TIMEOUT 3
#define MAX_NUM 10
#define COUNT_DATA 5

char *station_list[MAX_NUM];
char *job_list[MAX_NUM];
int station_num,job_num;

WINDOW	*win,*view_win,*top_win; /*view window*/


/*width of columns*/
int	column[COUNT_DATA]={18, 
			    12, 
			    11, 
			    9, 
			    10, 
			    };

/*-------------------------functions-----------------------*/

void print_data(int all,char *station,int x,int y)
{
long int mem[6]={0,0,0,0,0,0};
char tmpstr[MAX_LINE];

if ((all==0)&&(job_num==0)) return;

    /*set color*/
    init_pair(3,COLOR_YELLOW,COLOR_BLACK);
    wattrset(win, COLOR_PAIR(3)|A_BOLD);
    
    mvwaddstr(win,x,y,station);/*station*/
    
    /*set color*/
    init_pair(3,COLOR_YELLOW,COLOR_BLACK);
    wattrset(win, COLOR_PAIR(3));

if(job_num>0)
    sscanf(job_list[job_num-1],"%ld %ld %ld %ld %ld %ld",
	&mem[0],&mem[1],&mem[2],&mem[3],&mem[4],&mem[5]);
	
sprintf(tmpstr,"%10ld %10ld %10ld %10ld %10ld",
mem[0],mem[1],mem[2],mem[3],mem[4]);

mvwaddstr(win,x,y+13,tmpstr);
}

/*print menu */
void print_menu()
{

    int width  = COLS;
    int height = 2;                
    top_win = newwin(height, width, 0,0);
    
    return;
}

/*print head of table*/
void print_menu_top()
{
    int i=0,horizontal=1;
    char tmpstr[MAX_LINE];
    
    wclear(top_win);
    wrefresh(top_win);
    
    init_pair(1,COLOR_CYAN,COLOR_BLACK);
    wattrset(top_win, COLOR_PAIR(1)|A_BOLD);
    
    while (i<=COUNT_DATA)
    {

    switch (i){
	case 0: strncpy(tmpstr,"station",MAX_LINE);
		break;
	case 1: strncpy(tmpstr,"total",MAX_LINE);
		break;
	case 2: strncpy(tmpstr,"used",MAX_LINE);
		break;
	case 3: strncpy(tmpstr,"free",MAX_LINE);
		break;
	case 4: strncpy(tmpstr,"shared",MAX_LINE);
		break;
	case 5: strncpy(tmpstr,"buffers",MAX_LINE);
		break;
	case 6: strncpy(tmpstr,"cached",MAX_LINE);
		break;

	}
	mvwaddstr(top_win,0,horizontal,tmpstr);
	horizontal+=column[i];
	i++;
    }
    wrefresh(top_win);
}

/*light version of re-reading*/
void fill_stations()
{
    int		curr_point=0;
    
    /*clear window*/
    wclear(win);
    wrefresh(win);
    
    /*set color*/
    init_pair(3,COLOR_YELLOW,COLOR_BLACK);
    wattrset(win, COLOR_PAIR(3));
    
    /*draw items per screen*/
    for (curr_point=0;(curr_point<LINES-2)&&(curr_point<station_num);curr_point++)
         print_data(1,station_list[curr_point],curr_point,0);
    
    wrefresh(win);    
}


/*print view window*/
void print_view()
{
    int     width, height;

    /*view helper*/
    width  = COLS;
    height = LINES-2;                
    view_win = newwin(height, width, (LINES-height)/2, (COLS-width)/2);
    /*view main*/
    width  = COLS-2;
    height = LINES-4;
    win      = newwin(height, width, (LINES-height)/2, (COLS-width)/2);
    
    /*menu*/
    width  = COLS;
    height = 1;                
    top_win = newwin(height, width, 0,0);
    
    /*check for failure*/
    if ((win == NULL)||(view_win==NULL)||(top_win==NULL))
    {   
        endwin();
        return;
    }
    
    /*print top menu*/
    print_menu_top();
    
    init_pair(2,COLOR_CYAN,COLOR_BLACK);
    
    wattrset(view_win, COLOR_PAIR(2));
    wattrset(win, COLOR_PAIR(2));
    
    box(view_win, ACS_VLINE, ACS_HLINE);
    waddstr(view_win,"R-monitor (Memory usage)");
    

    wrefresh(top_win);
    wrefresh(view_win);
    wrefresh(win);
    return;
}

/*Main function*/
int main(int argc, char *argv[])
{
    char command_str[MAX_LINE],
    keyword[MAX_LINE],session_keyword[MAX_LINE];
    
        
    int sleep_time=SLEEP_TIMEOUT;
    int s,ch;

    char *hostname=NULL;

    for(ch=0;ch<MAX_NUM;ch++) station_list[ch]=malloc(MAX_LINE);
    for(ch=0;ch<MAX_NUM;ch++) job_list[ch]=malloc(MAX_LINE);

    /*check args*/
    while ((ch = getopt(argc, argv, "h:t:")) != EOF)
	switch((char)ch)
	{
	    case 't':		/* Local User */
		    sleep_time = strtol(optarg,0,0);
		    break;
	    case 'h':           /*Host for connection*/
		    hostname=optarg;
		    break;
	}

    /*analize options*/
    if (hostname == NULL) {
        hostname=malloc(MAX_LINE);
        strncpy(hostname,DEFAULT_HOST,MAX_LINE);
    }
    
    /*get keyword*/
    get_keyword(keyword,1);    
    

    /*Stage 1: get stations list*/
    
    /*make command_str*/
    strncpy(command_str,LIST_STATION_COMMAND,MAX_LINE); /*We add data to itself job-list*/
    /*create connection*/
    s=client_connection(hostname);
    /*send password*/
    client_check_keyword(s,keyword,session_keyword);
    /*send command to server*/
    client_write_command(s,command_str);
    
    /*read answer*/
    station_num=client_read_message_v(s,session_keyword,station_list);
    
    close(s);
    
#ifdef TEST
//    for(ch=0;ch<station_num;ch++) fprintf(stderr,">>%s\n",station_list[ch]);
#endif

/*---------------------CURSES--------------------*/
    /*init curses*/
    initscr();
    start_color();

    /*print menu and view*/
    print_menu();
    print_view();    
    fill_stations();
/*---------------------CURSES--------------------*/


    /*Stage 2:main working cicle*/
    
    while(1){    
    	for(ch=0;ch<station_num;ch++){
	    /*make command_str*/
	    strncpy(command_str,GET_ONLY_ONE_STATION_COMMAND,MAX_LINE); /*We get data from selected station*/
	    /*create connection*/
	    s=client_connection(hostname);
	    /*send password*/
	    client_check_keyword(s,keyword,session_keyword);
	    /*send command to server*/
	    client_write_command(s,command_str);
	    /*select station*/
	    client_write_message(s,station_list[ch],session_keyword);
	    /*get jobs to this station*/
	    job_num=client_read_message_v(s,session_keyword,job_list);
	    
	    close(s);
	    
	    print_data(0,station_list[ch],ch,0);
	
	}/*for*/
    wrefresh(win);
    sleep(3);
    }/*while*/
    
    /*finish work*/
    for(ch=0;ch<MAX_NUM;ch++) free(job_list[ch]);
    for(ch=0;ch<MAX_NUM;ch++) free(station_list[ch]);
    return(0);
}